<?php

require_once('config.php');
require_once('mySocketServer.php');

class myChatServer extends mySocketServer
{
  protected
    $userAction = array('userConnect', 'userSendMessage'),
    $listUser = array(),
    $listRoom = array();

  protected function onStart()
  {
    $this->last_activity = time();
    $this->lifetime = 60;
    $this->log("myChatServer->onStart()");
  }

  protected function onRunCycle()
  {
    $this->log("last_activity+lifime: ".($this->last_activity + $this->lifetime)." time: ".time());

    if($this->last_activity + $this->lifetime < time())
    {
      $this->log("myChatServer lifetime expired");
      $this->shutdown();
    }
  }

  protected function onShutdown()
  {
    $this->log("myChatServer->onShutdown()");
  }

  public function onReceiveData($id, $data)
  {
    $data = unserialize($data);

    //  Demande d'arrêt du serveur - a modifier pour empecher n'importe qui de le faire
    if(isset($data['msg']) && strstr($data['msg'], '/shutdown'))
    {
      $this->shutdown();
      return;
    }

    //  La connexion ne provient pas de notre serveur
    if($this->clientInfo[$id]['host'] != CHAT_IP)
    {
      $this->log('ip non autorisé');
      return;
    }

    //  Action non autorisé par le serveur de chat
    if( ! in_array($data['action'], $this->userAction))
    {
      $this->log('action non autorisé');
      return;
    }

    $this->log('listUser: '.print_r($this->listUser, true));
    $this->log('listRoom: '.print_r($this->listRoom, true));

    $this->last_activity = time();

    //  Execution de l'action
    $this->$data['action']($id, $data);
  }

  protected function userConnect($id, $data)
  {
    $renewConnect = false;

    //  Verifier si le username n'est pas déjà utilisé
    if(($clientId = $this->checkUserExist('username', $data)) !== false)
    {
      $this->log('utilisateur existant, suppr ancienne référence dans listUser');

      $roomId = $this->listUser[$clientId]['userroom'];

      if(isset($this->listRoom[$roomId][$clientId]))
      {
        $this->log('utilisateur existant, suppr ancienne référence dans listRoom');
        unset($this->listRoom[$roomId][$clientId]);
      }

      unset($this->listUser[$clientId]);

      $renewConnect = true;
    }

    //  On stocke les infos de l'utilisateur
    $this->listUser[$id] = array(
      'username'=>$data['username'],
      'userroom'=>$data['room'],
      'userip'=>$data['userip'],
      'userid'=>$data['userid']
    );

    //  La salle n'existe pas, on la créé
    if(!isset($this->listRoom[$data['room']]))
    {
      $this->listRoom[$data['room']] = array();
    }

    //  L'utilisateur n'est pas encore référencé dans la salle, on l'ajoute
    if(!in_array($data['username'], $this->listRoom[$data['room']]))
    {
      $this->listRoom[$data['room']][$id] = $data['username'];
    }

    //  Envoie du message à la room concerné si ce n'est pas un renouvellement de connexion
    if(!$renewConnect)
    {
      $msgRoom = '<p>'.$data['username'].' vient de se connecter</p>'.$this->userGetRoomUser($data['room']);
      $msgRoom = self::format_message($msgRoom);
      $this->log($msgRoom.' - room: '.$data['room']);

      $this->broadcastDataToRoom($data['room'], $msgRoom, $id);

      //  Envoie du message à l'utilisateur qui vient de se connecter
      $msgUser = '<p>Bienvenue dans la salle '.$data['room'].'</p>'.$this->userGetRoomUser($data['room']);
      $msgUser = self::format_message($msgUser);
      $this->sendData($id, $msgUser);
    }
    else
    {
      $msgRoom = $this->userGetRoomUser($data['room']).$this->readEndCharacter;
      $this->broadcastDataToRoom($data['room'], $msgRoom);
    }
    //
  }

  protected function userSendMessage($id, $data)
  {
    $this->log('userSendMessage');

    //  La salle n'est pas renseigné ou n'existe pas
    if(!isset($data['room'], $this->listRoom[$data['room']]))
    {
      $this->log('userSendMessage - la salle n\'est pas renseigné ou n\'existe pas');
      return;
    }

    //  On prépare le message a transmettre
    $msg = '<p><span class="username">'.$data['username'].'</span>: '.strip_tags($data['msg']).'</p>';
    $msg = self::format_message($msg);

    //  On transmet le message au client connecté à la room
    $this->broadcastDataToRoom($data['room'], $msg);
  }

  protected function userGetRoomUser($roomId)
  {
    $this->log('userGetRoomUser');

    $msg = '<div class="chatlistuser" style="display:none;"><ul>';

    $room = $this->listRoom[$roomId];
    foreach($room as $clientId=>$username)
    {
      if(isset($this->listUser[$clientId]))
        $msg .= '<li>'.$this->listUser[$clientId]['username'].'</li>';
    }

    $msg .= '</ul></div>'.$this->readEndCharacter;

    return $msg;
  }

  protected function checkUserExist($key, $data)
  {
    foreach($this->listUser as $clientId=>$user)
    {
      $this->log('checkUserExist::comparaison( '.$user[$key].' == '.$data[$key].' )');

      if($user[$key] == $data[$key])
      {
        return $clientId;
        break;
      }
    }

    return false;
  }

  protected function broadcastDataToRoom($room, $data, $exclude=array())
  {
    if( ! is_array($exclude))
    {
      $exclude = array($exclude);
    }

    $room = $this->listRoom[$room];
    foreach($room as $clientId=>$username)
    {
      if(isset($this->listClient[$clientId]) && $this->listClient[$clientId] != NULL && ! in_array($clientId, $exclude))
      {
        $this->sendData($clientId, $data);
      }
    }
  }

  public static function format_message($message)
  {
    $msg = '<div class="message" id="'.md5(microtime()).'">'.$message.'</div>'."\n";

    return $msg;
  }

  protected function onConnect($id)
  {

  }

  protected function onConnectionRefused($id)
  {

  }

  protected function onCloseConnection($id)
  {
    if(isset($this->listUser[$id]))
    {
      $roomId = $this->listUser[$id]['userroom'];

      if(isset($this->listRoom[$roomId][$id]))
      {
        unset($this->listRoom[$roomId][$id]);
      }

      $msg = '<p>'.$this->listUser[$id]['username'].' s\'est déconnecté du chat</p>'.$this->userGetRoomUser($roomId);
      $msg = self::format_message($msg);

      $this->log($msg);
      $this->broadcastDataToRoom($this->listUser[$id]['userroom'], $msg, $id);

      unset($this->listUser[$id]);
    }
  }

  protected function log($msg)
  {
    parent::log($msg);
  }
}
